#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e13|e14|e15|e16|e17|e19|c1> <debug|opt|prof> [tar]"
}
# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi


# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=net_snmp
origpkgver=v5_9_0
pkgver=${origpkgver}
ssibuildshims_version=v2_09_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarname='net-snmp-5.9'
#pkgtarname=net-snmp-${pkgdotver}
#pkgtarfile=${pkgtarname}.tgz

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} NO_QUAL
   exit 0
fi

echo "building ${package} for ${OS}-${plat} (flavor ${flvr})"


# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "opt" ]
then
    cxxflg="-O3 -g -DNDEBUG"
elif [ "${extraqual}" = "debug" ]
then
    cxxflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cxxflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e[79] ]]
then
  xcc=gcc
  xcpc=g++
  cxxflg="${cxxflg} -std=c++1y -Wno-deprecated-declarations"
elif [[ "${basequal}" == e1[045] ]]
then
  xcc=gcc
  xcpc=g++
  cxxflg="${cxxflg} -std=c++14 -Wno-deprecated-declarations"
elif [[ "${basequal}" == e1[679] ]]
then
  xcc=gcc
  xcpc=g++
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == e13 ]]
then
  xcc=clang
  xcpc=clang++
  cxxflg="${cxxflg} -std=c++14 -Wno-deprecated-declarations"
elif [[ "${basequal}" == c1 ]]
then
  xcc=clang
  xcpc=clang++
  cxxflg="${cxxflg} -std=c++17 -Wno-deprecated-declarations -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
else
  ssi_die "Qualifier $basequal not recognized."
fi

mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "Fake declaration failed."

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${PRODUCTS} || ssi_die "fake setup failed"

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${sourcedir}
tar xf ${tardir}/${pkgtarname}.tar.gz

# patch
patchfile=${patchdir}/net_snmp.patch

if [ -f "${patchfile}" ]
then
  cd ${sourcedir}/${pkgtarname}
  patch --verbose -b -p0 < ${patchdir}/net_snmp.patch || ssi_die "ERROR: patch failed."
fi

cd ${blddir}

# Don't check configure status -- it's buggy.
if [[ ${OS} == "rh7-ppc64le" ]];then
CC=${xcc} CXX=${xcpc} CXXFLAGS="${cxxflg}" ${sourcedir}/${pkgtarname}/configure \
    --prefix=${pkgdir} --build="powerpc64-unknown-linux-gnu" \
    --with-sys-contact="@@no.where" \
    --with-default-snmp-version="3" \
    --with-sys-location="artdaq_host" \
    --with-logfile="/var/log/snmpd.log" \
    --with-persistent-directory="/var/net-snmp"
    
else
CC=${xcc} CXX=${xcpc} CXXFLAGS="${cxxflg}" ${sourcedir}/${pkgtarname}/configure --prefix=${pkgdir} \
    --with-sys-contact="@@no.where" \
    --with-default-snmp-version="3" \
    --with-sys-location="artdaq_host" \
    --with-logfile="/var/log/snmpd.log" \
    --with-persistent-directory="/var/net-snmp"
fi

#make -j ${ncore} || ssi_die "make failed."
make -j || ssi_die "make failed."
make install || ssi_die "make install failed."

set +x

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "net_snmp libraries are in ${NET_SNMP_LIB}"

# fixing embedded path
mv ${NET_SNMP_FQ_DIR}/bin/net-snmp-config ${NET_SNMP_FQ_DIR}/bin/net_snmp-config.bak
cat ${NET_SNMP_FQ_DIR}/bin/net_snmp-config.bak | sed 's%^prefix=.*$%prefix=${NET_SNMP_FQ_DIR}%' > ${NET_SNMP_FQ_DIR}/bin/net_snmp-config
chmod +x ${NET_SNMP_FQ_DIR}/bin/net_snmp-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
